@@ -54,6 +54,7 @@ iBool   isAlphaNumeric_Char (iChar);
 iBool   isPunct_Char        (iChar);
 
 const char *script_Char     (iChar);
+int     width_Char          (iChar);
 
 iLocalDef iBool isVariationSelector_Char(iChar c) {
     return (c >= 0xfe00 && c <= 0xfe0f) || (c >= 0xe0100 && c <= 0xe0121);
@@ -67,7 +68,9 @@ iLocalDef iBool isDefaultIgnorable_Char(iChar c) {
 }
 iLocalDef iBool isEmoji_Char(iChar c) {
     return (c >= 0x1f300 && c < 0x1f700) || (c >= 0x1f7e0 && c <= 0x1f7eb) ||
-           (c >= 0x1f900 && c <= 0x1f9ff) || (c >= 0x1fa70 && c <= 0x1faff);
+           (c >= 0x1f900 && c <= 0x1f9ff) || (c >= 0x1fa70 && c <= 0x1faff) ||
+           c == 0x2728 || c == 0x270a || c == 0x270b || c == 0x274c || c == 0x274e ||
+           (c >= 0x26f0 && c <= 0x26fa);
 }
 iLocalDef iBool isDingbats_Char(iChar c) {
     return c >= 0x2702 && c <= 0x27b0;
@@ -76,7 +79,7 @@ iLocalDef iBool isPictograph_Char(iChar c) {
     return (c == 0x21a9) ||
            (c == 0x2218 || c == 0x2219) ||
            (c >= 0x2300 && c <= 0x27bf) ||
-            c == 0x2a2f ||           
+            c == 0x2a2f ||
            (c >= 0x2b9c && c <= 0x2b9f) /* arrowheads */ ||
            (c >= 0x1f680 && c <= 0x1f6c0);
 }
@@ -154,6 +157,8 @@ iString *       upperLang_String    (const iString *, const char *langCode);
 iString *       lower_String        (const iString *);
 iString *       lowerLang_String    (const iString *, const char *langCode);
 iStringList *   split_String        (const iString *, const char *separator);
+iString *       concat_String       (const iString *, const iString *other);
+iString *       concatCStr_String   (const iString *, const char *other);
 iChar           first_String        (const iString *);
 iChar           last_String         (const iString *);
 iBlock *        toLocal_String      (const iString *);
@@ -187,12 +192,24 @@ iLocalDef const char *cstrCollect_String(iString *d) {
 
 iLocalDef iRangecc range_String(const iString *d) {
     const iRangecc r = {
-        d ? constBegin_Block(&(d)->chars) : NULL,
-        d ? constEnd_Block(&(d)->chars) : NULL
+        d ? constBegin_Block(&d->chars) : NULL,
+        d ? constEnd_Block(&d->chars) : NULL
     };
     return r;
 }
 
+iLocalDef iRangecc rangeN_String(const iString *d, size_t n) {
+    if (!d) return (iRangecc) { NULL, NULL };
+    return (iRangecc) { constBegin_Block(&d->chars),
+                        constBegin_Block(&d->chars) + iMin(n, size_Block(&d->chars)) };
+}
+
+iLocalDef iRangecc rangeFrom_String(const iString *d, size_t from) {
+    if (!d) return (iRangecc) { NULL, NULL };
+    return (iRangecc) { constBegin_Block(&d->chars) + iMin(from, size_Block(&d->chars)),
+                        constEnd_Block(&d->chars) };
+}
+
 iLocalDef const iBlock *utf8_String(const iString *d) {
     return &d->chars; /* unmodified internal representation (UTF-8) */
 }
@@ -298,7 +315,10 @@ float           toFloat_String  (const iString *);
 double          toDouble_String (const iString *);
 
 iString *       quote_String    (const iString *, iBool numericUnicode);
-iString *       unquote_String  (const iString *);
+iString *       unquote_String  (const iString *); /* delimiter is `"` */
+iString *       unquoteDelim_String (const iString *, iChar delim); /* custom delimiter */
+
+void            setLocaleCharSet_String (const char *charSet); /* charset for initLocal, toLocal */
 
 const char *    format_CStr     (const char *format, ...);
 const char *    skipSpace_CStr  (const char *);
@@ -357,6 +377,7 @@ iLocalDef iBool endsWithCase_Rangecc(const iRangecc d, const char *cstr) {
     return endsWithSc_Rangecc(d, cstr, &iCaseInsensitive);
 }
 
+iBlock *        toLocal_Rangecc     (iRangecc);
 iStringList *   split_Rangecc       (iRangecc, const char *separator);
 void            trimStart_Rangecc   (iRangecc *);
 void            trimEnd_Rangecc     (iRangecc *);
