@@ -50,12 +50,16 @@ iDeclareTypeConstruction(Visited)
 void    clear_Visited           (iVisited *);
 void    load_Visited            (iVisited *, const char *dirPath);
 void    save_Visited            (const iVisited *, const char *dirPath);
+void    serialize_Visited       (const iVisited *, iStream *out);
+void    deserialize_Visited     (iVisited *, iStream *ins, iBool mergeKeepingLatest);
 
 iTime   urlVisitTime_Visited    (const iVisited *, const iString *url);
 void    visitUrl_Visited        (iVisited *, const iString *url, uint16_t visitFlags); /* adds URL to the visited URLs set */
+void    visitUrlTime_Visited    (iVisited *, const iString *url, uint16_t visitFlags, iTime when);
 void    setUrlKept_Visited      (iVisited *, const iString *url, iBool isKept); /* URL is marked as (non)discardable */
 void    removeUrl_Visited       (iVisited *, const iString *url);
 iBool   containsUrl_Visited     (const iVisited *, const iString *url);
 
 const iPtrArray *   list_Visited        (const iVisited *, size_t count); /* returns collected */
+const iPtrArray *   listMatching_Visited(const iVisited *, const char *prefix);
 const iPtrArray *   listKept_Visited    (const iVisited *);
